@@ -90,6 +90,8 @@ ReceiverURLs = namedtuple(
         "command_hdmi_audio_decode",
         "command_quick_select_mode",
         "command_quick_select_memory",
+        "command_smart_select_mode",  # Marantz
+        "command_smart_select_memory",  # Marantz
         "command_auto_standby",
         "command_sleep",
         "command_center_spread",
@@ -183,6 +185,8 @@ TelnetCommands = namedtuple(
         "command_hdmi_audio_decode",
         "command_quick_select_mode",
         "command_quick_select_memory",
+        "command_smart_select_mode",  # Marantz
+        "command_smart_select_memory",  # Marantz
         "command_auto_standby",
         "command_sleep",
         "command_center_spread",
@@ -364,9 +368,13 @@ SOUND_MODE_MAPPING = {
         "DOLBY AUDIO - TRUEHD + NEURAL:X",
         "DOLBY AUDIO-DD",
         "DOLBY AUDIO-DD+ +DSUR",
+        # Added both variants as we don't know if type is intentional
+        "DOLBY AUDIO-DD+ +NERUAL:X",
         "DOLBY AUDIO-DD+ +NEURAL:X",
         "DOLBY AUDIO-DD+",
+        "DOLBY AUDIO-DD+",
         "DOLBY AUDIO-DD+DSUR",
+        "DOLBY AUDIO-DD+NEURAL:X",
         "DOLBY AUDIO-DSUR",
         "DOLBY AUDIO-TRUEHD",
         "DOLBY AUDIO-TRUEHD+DSUR",
@@ -391,16 +399,16 @@ SOUND_MODE_MAPPING = {
         "MULTI IN + NEURAL:X",
         "NEURAL",
         "STANDARD(DOLBY)",
-        "DOLBY AUDIO-DD+NEURAL:X",
-        "DOLBY AUDIO-DD+",
-        # Added both variants as we don't know if type is intentional
-        "DOLBY AUDIO-DD+ +NERUAL:X",
-        "DOLBY AUDIO-DD+ +NEURAL:X",
     ],
     "DTS SURROUND": [
         "DTS + DOLBY SURROUND",
         "DTS + NEURAL:X",
         "DTS + VIRTUAL:X",
+        "DTS HD MSTR",
+        "DTS HD",
+        "DTS HD+DSUR",
+        "DTS HD+NEURAL:X",
+        "DTS HD+VIRTUAL:X",
         "DTS NEURAL:X",
         "DTS SURROUND",
         "DTS VIRTUAL:X",
@@ -411,26 +419,22 @@ SOUND_MODE_MAPPING = {
         "DTS-HD",
         "DTS:X MSTR",
         "DTS:X",
+        "DTS:X+VIRTUAL:X",
+        "DTS+DSUR",
+        "DTS+NEURAL:X",
+        "DTS+VIRTUAL:X",
+        "IMAX DTS:X",
+        "IMAX DTS:X+VIRTUAL:X",
+        "IMAX DTS",
+        "IMAX DTS+NEURAL:X",
         "M CH IN+DSUR",
+        "M CH IN+NEURAL:X",
+        "M CH IN+VIRTUAL:X",
+        "MAX DTS+VIRTUAL:X",
         "MULTI CH IN",
         "NEURAL:X",
         "STANDARD(DTS)",
         "VIRTUAL:X",
-        "IMAX DTS",
-        "IMAX DTS+NEURAL:X",
-        "MAX DTS+VIRTUAL:X",
-        "DTS+DSUR",
-        "DTS+NEURAL:X",
-        "DTS+VIRTUAL:X",
-        "DTS HD",
-        "DTS HD+DSUR",
-        "DTS HD+NEURAL:X",
-        "DTS HD+VIRTUAL:X",
-        "DTS:X+VIRTUAL:X",
-        "IMAX DTS:X",
-        "IMAX DTS:X+VIRTUAL:X",
-        "M CH IN+NEURAL:X",
-        "M CH IN+VIRTUAL:X",
     ],
     "AURO3D": ["AURO-3D", "AURO3D"],
     "AURO2DSURR": ["AURO-2D SURROUND", "AURO2DSURR"],
@@ -541,7 +545,11 @@ COMMAND_DIALOG_ENHANCER = "/goform/formiPhoneAppDirect.xml?PSDEH%20{level}"
 COMMAND_HDMI_OUTPUT = "/goform/formiPhoneAppDirect.xml?VSMONI{output}"
 COMMAND_HDMI_AUDIO_DECODE = "/goform/formiPhoneAppDirect.xml?VSAUDIO%20{mode}"
 COMMAND_QUICK_SELECT_MODE = "/goform/formiPhoneAppDirect.xml?MSQUICK{number}"
-COMMAND_QUICK_SELECT_MEMORY = "/goform/formiPhoneAppDirect.xml?MSQUICK{number}"
+COMMAND_QUICK_SELECT_MEMORY = "/goform/formiPhoneAppDirect.xml?MSQUICK{number}%20MEMORY"
+COMMAND_SMART_SELECT_MODE = "/goform/formiPhoneAppDirect.xml?MSSMART{number}"  # Marantz
+COMMAND_SMART_SELECT_MEMORY = (
+    "/goform/formiPhoneAppDirect.xml?MSSMART{number}%20MEMORY"  # Marantz
+)
 COMMAND_AUTO_STANDBY = "/goform/formiPhoneAppDirect.xml?STBY{mode}"
 COMMAND_SLEEP = "/goform/formiPhoneAppDirect.xml?SLP{value}"
 COMMAND_CENTER_SPREAD = "/goform/formiPhoneAppDirect.xml?PSCES%20{mode}"
@@ -658,6 +666,8 @@ DENONAVR_URLS = ReceiverURLs(
     command_hdmi_audio_decode=COMMAND_HDMI_AUDIO_DECODE,
     command_quick_select_mode=COMMAND_QUICK_SELECT_MODE,
     command_quick_select_memory=COMMAND_QUICK_SELECT_MODE,
+    command_smart_select_mode=COMMAND_SMART_SELECT_MODE,  # Marantz
+    command_smart_select_memory=COMMAND_SMART_SELECT_MEMORY,  # Marantz
     command_auto_standby=COMMAND_AUTO_STANDBY,
     command_sleep=COMMAND_SLEEP,
     command_center_spread=COMMAND_CENTER_SPREAD,
@@ -751,6 +761,8 @@ ZONE2_URLS = ReceiverURLs(
     command_hdmi_audio_decode=COMMAND_HDMI_AUDIO_DECODE,
     command_quick_select_mode=COMMAND_QUICK_SELECT_MODE,
     command_quick_select_memory=COMMAND_QUICK_SELECT_MEMORY,
+    command_smart_select_mode=COMMAND_SMART_SELECT_MODE,  # Marantz
+    command_smart_select_memory=COMMAND_SMART_SELECT_MEMORY,  # Marantz
     command_auto_standby=COMMAND_AUTO_STANDBY,
     command_sleep=COMMAND_SLEEP,
     command_center_spread=COMMAND_CENTER_SPREAD,
@@ -844,6 +856,8 @@ ZONE3_URLS = ReceiverURLs(
     command_hdmi_audio_decode=COMMAND_HDMI_AUDIO_DECODE,
     command_quick_select_mode=COMMAND_QUICK_SELECT_MODE,
     command_quick_select_memory=COMMAND_QUICK_SELECT_MEMORY,
+    command_smart_select_mode=COMMAND_SMART_SELECT_MODE,  # Marantz
+    command_smart_select_memory=COMMAND_SMART_SELECT_MEMORY,  # Marantz
     command_auto_standby=COMMAND_AUTO_STANDBY,
     command_sleep=COMMAND_SLEEP,
     command_center_spread=COMMAND_CENTER_SPREAD,
@@ -992,6 +1006,8 @@ DENONAVR_TELNET_COMMANDS = TelnetCommands(
     command_hdmi_audio_decode="VSAUDIO {mode}",
     command_quick_select_mode="MSQUICK{number}",
     command_quick_select_memory="MSQUICK{number} MEMORY",
+    command_smart_select_mode="MSSMART{number}",  # Marantz
+    command_smart_select_memory="MSSMART{number} MEMORY",  # Marantz
     command_auto_standby="STBY{mode}",
     command_sleep="SLP{value}",
     command_center_spread="PSCES {mode}",
@@ -1083,6 +1099,8 @@ ZONE2_TELNET_COMMANDS = TelnetCommands(
     command_hdmi_audio_decode="VSAUDIO {mode}",
     command_quick_select_mode="MSQUICK{number}",
     command_quick_select_memory="MSQUICK{number} MEMORY",
+    command_smart_select_mode="MSSMART{number}",  # Marantz
+    command_smart_select_memory="MSSMART{number} MEMORY",  # Marantz
     command_auto_standby="STBY{mode}",
     command_sleep="SLP{value}",
     command_center_spread="PSCES {mode}",
@@ -1174,6 +1192,8 @@ ZONE3_TELNET_COMMANDS = TelnetCommands(
     command_hdmi_audio_decode="VSAUDIO {mode}",
     command_quick_select_mode="MSQUICK{number}",
     command_quick_select_memory="MSQUICK{number} MEMORY",
+    command_smart_select_mode="MSSMART{number}",  # Marantz
+    command_smart_select_memory="MSSMART{number} MEMORY",  # Marantz
     command_auto_standby="STBY{mode}",
     command_sleep="SLP{value}",
     command_center_spread="PSCES {mode}",
