@@ -10,14 +10,29 @@ This module implements the interface to Denon AVR receivers.
 import asyncio
 import logging
 import time
-from typing import Awaitable, Callable, Dict, List, Optional
+from typing import Awaitable, Callable, Dict, List, Literal, Optional, Union
 
 import attr
 import httpx
 
 from .audyssey import DenonAVRAudyssey, audyssey_factory
-from .const import DENON_ATTR_SETATTR, MAIN_ZONE, VALID_ZONES
-from .decorators import run_async_synchronously
+from .const import (
+    DENON_ATTR_SETATTR,
+    MAIN_ZONE,
+    VALID_ZONES,
+    AudioRestorers,
+    AutoStandbys,
+    BluetoothOutputModes,
+    DimmerModes,
+    EcoModes,
+    HDMIAudioDecodes,
+    HDMIOutputs,
+    PanelLocks,
+    RoomSizes,
+    TransducerLPFs,
+    VideoProcessingModes,
+)
+from .dirac import DenonAVRDirac, dirac_factory
 from .exceptions import AvrCommandError
 from .foundation import DenonAVRFoundation, set_api_host, set_api_timeout
 from .input import DenonAVRInput, input_factory
@@ -84,6 +99,11 @@ class DenonAVR(DenonAVRFoundation):
         default=attr.Factory(audyssey_factory, takes_self=True),
         init=False,
     )
+    dirac: DenonAVRDirac = attr.ib(
+        validator=attr.validators.instance_of(DenonAVRDirac),
+        default=attr.Factory(dirac_factory, takes_self=True),
+        init=False,
+    )
     input: DenonAVRInput = attr.ib(
         validator=attr.validators.instance_of(DenonAVRInput),
         default=attr.Factory(input_factory, takes_self=True),
@@ -124,7 +144,7 @@ class DenonAVR(DenonAVRFoundation):
             # Name either set explicitly or name of Main Zone with suffix
             zonename = None
             if zname is None and self._name is not None:
-                zonename = "{} {}".format(self._name, zone)
+                zonename = f"{self._name} {zone}"
             zone_device = attr.evolve(self._device, zone=zone)
             zone_inst = DenonAVR(
                 host=self._host,
@@ -138,6 +158,7 @@ class DenonAVR(DenonAVRFoundation):
     async def async_setup(self) -> None:
         """Ensure that configuration is loaded from receiver asynchronously."""
         async with self._setup_lock:
+            _LOGGER.debug("Starting denonavr setup")
             # Device setup
             await self._device.async_setup()
             if self._name is None:
@@ -146,19 +167,17 @@ class DenonAVR(DenonAVRFoundation):
             # Setup other functions
             self.input.setup()
             await self.soundmode.async_setup()
-            self.tonecontrol.setup()
+            await self.tonecontrol.async_setup()
             self.vol.setup()
             self.audyssey.setup()
+            self.dirac.setup()
 
             for zone_name, zone_item in self._zones.items():
                 if zone_name != self.zone:
                     await zone_item.async_setup()
 
             self._is_setup = True
-
-    @run_async_synchronously(async_func=async_setup)
-    def setup(self) -> None:
-        """Ensure that configuration is loaded from receiver."""
+            _LOGGER.debug("Finished denonavr setup")
 
     async def async_update(self):
         """
@@ -166,14 +185,16 @@ class DenonAVR(DenonAVRFoundation):
 
         Method executes the update method for the current receiver type.
         """
+        _LOGGER.debug("Starting denonavr update")
         # Ensure that the device is setup
-        if self._is_setup is False:
+        if not self._is_setup:
             await self.async_setup()
 
         # Create a cache id for this global update
         cache_id = time.time()
 
         # Verify update method
+        _LOGGER.debug("Verifying update method")
         await self._device.async_verify_avr_2016_update_method(cache_id=cache_id)
 
         # Update device
@@ -189,42 +210,23 @@ class DenonAVR(DenonAVRFoundation):
         # into main update
         # await self.audyssey.async_update(
         #     global_update=True, cache_id=cache_id)
-
-    @run_async_synchronously(async_func=async_update)
-    def update(self):
-        """
-        Get the latest status information from device.
-
-        Method executes the update method for the current receiver type.
-        """
+        _LOGGER.debug("Finished denonavr update")
 
     async def async_update_tonecontrol(self):
         """Get Tonecontrol settings."""
         await self.tonecontrol.async_update()
 
-    @run_async_synchronously(async_func=async_update_tonecontrol)
-    def update_tonecontrol(self):
-        """Get Tonecontrol settings."""
-
     async def async_update_audyssey(self):
         """Get Audyssey settings."""
         await self.audyssey.async_update()
 
-    @run_async_synchronously(async_func=async_update_audyssey)
-    def update_audyssey(self):
-        """Get Audyssey settings."""
-
     async def async_get_command(self, request: str) -> str:
         """Send HTTP GET command to Denon AVR receiver asynchronously."""
         return await self._device.api.async_get_command(request)
 
-    @run_async_synchronously(async_func=async_get_command)
-    def get_command(self, request: str) -> str:
-        """Send HTTP GET command to Denon AVR receiver."""
-
-    @run_async_synchronously(async_func=async_get_command)
-    def send_get_command(self, request: str) -> str:
-        """Send HTTP GET command to Denon AVR receiver...for compatibility."""
+    async def async_send_telnet_commands(self, *commands: str) -> None:
+        """Send telnet commands to the receiver."""
+        await self._device.telnet_api.async_send_commands(*commands)
 
     def send_telnet_commands(self, *commands: str) -> None:
         """Send telnet commands to the receiver."""
@@ -298,6 +300,15 @@ class DenonAVR(DenonAVRFoundation):
         """
         return self._device.power
 
+    @property
+    def settings_menu(self) -> Optional[bool]:
+        """
+        Return the settings menu state of the device.
+
+        Only available if using Telnet.
+        """
+        return self._device.settings_menu
+
     @property
     def state(self) -> Optional[str]:
         """
@@ -340,7 +351,7 @@ class DenonAVR(DenonAVRFoundation):
 
     @property
     def support_sound_mode(self) -> Optional[bool]:
-        """Return True if sound mode supported."""
+        """Return True if sound mode is supported."""
         return self.soundmode.support_sound_mode
 
     @property
@@ -435,13 +446,18 @@ class DenonAVR(DenonAVRFoundation):
             return None
         return self._device.receiver.type
 
+    @property
+    def telnet_available(self) -> bool:
+        """Return True if telnet is connected and healthy."""
+        return self._device.telnet_available
+
     @property
     def telnet_connected(self) -> bool:
         """Return True if telnet is connected."""
         return self._device.telnet_api.connected
 
     @property
-    def telnet_healthy(self) -> Optional[bool]:
+    def telnet_healthy(self) -> bool:
         """Return True if telnet connection is healthy."""
         return self._device.telnet_api.healthy
 
@@ -450,6 +466,21 @@ class DenonAVR(DenonAVRFoundation):
         """Indicate if all inputs are shown or just active one."""
         return self._show_all_inputs
 
+    @property
+    def support_tone_control(self) -> Optional[bool]:
+        """Return True if tone control is supported."""
+        return self.tonecontrol.support_tone_control
+
+    @property
+    def tone_control_status(self) -> Optional[bool]:
+        """Return value of tone control status."""
+        return self.tonecontrol.tone_control_status
+
+    @property
+    def tone_control_adjust(self) -> Optional[bool]:
+        """Return value of tone control adjust."""
+        return self.tonecontrol.tone_control_adjust
+
     @property
     def bass(self) -> Optional[int]:
         """Return value of bass."""
@@ -505,9 +536,221 @@ class DenonAVR(DenonAVRFoundation):
         """Return a list of available MultiEQ settings."""
         return self.audyssey.multi_eq_setting_list
 
-    async def async_dynamic_eq_off(self) -> None:
-        """Turn DynamicEQ off."""
-        await self.audyssey.async_dynamiceq_off()
+    @property
+    def dimmer(self) -> Optional[str]:
+        """
+        Returns the dimmer state of the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "Off", "Dark", "Dim" and "Bright"
+        """
+        return self._device.dimmer
+
+    @property
+    def auto_standby(self) -> Optional[str]:
+        """
+        Return the auto-standby state of the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "OFF", "15M", "30M", "60M"
+        """
+        return self._device.auto_standby
+
+    @property
+    def sleep(self) -> Optional[Union[str, int]]:
+        """
+        Return the sleep timer for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "OFF" and 1-120 (in minutes)
+        """
+        return self._device.sleep
+
+    @property
+    def delay(self) -> Optional[int]:
+        """
+        Return the audio delay for the device in ms.
+
+        Only available if using Telnet.
+        """
+        return self._device.delay
+
+    @property
+    def eco_mode(self) -> Optional[str]:
+        """
+        Returns the eco-mode for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "Off", "On", "Auto"
+        """
+        return self._device.eco_mode
+
+    @property
+    def hdmi_output(self) -> Optional[str]:
+        """
+        Returns the HDMI-output for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "Auto", "HDMI1", "HDMI2"
+        """
+        return self._device.hdmi_output
+
+    @property
+    def hdmi_audio_decode(self) -> Optional[str]:
+        """
+        Returns the HDMI Audio Decode mode for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "AMP", "TV"
+        """
+        return self._device.hdmi_audio_decode
+
+    @property
+    def video_processing_mode(self) -> Optional[str]:
+        """
+        Return the video processing mode for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "Auto", "Game", "Movie", "Bypass"
+        """
+        return self._device.video_processing_mode
+
+    @property
+    def tactile_transducer(self) -> Optional[bool]:
+        """
+        Return the tactile transducer state of the device.
+
+        Only available if using Telnet.
+        """
+        return self._device.tactile_transducer
+
+    @property
+    def tactile_transducer_level(self) -> Optional[float]:
+        """
+        Return the tactile transducer level in dB.
+
+        Only available if using Telnet.
+        """
+        return self._device.tactile_transducer_level
+
+    @property
+    def tactile_transducer_lpf(self) -> Optional[str]:
+        """
+        Return the tactile transducer low pass filter frequency.
+
+        Only available if using Telnet.
+        """
+        return self._device.tactile_transducer_lpf
+
+    @property
+    def room_size(self) -> Optional[str]:
+        """
+        Return the room size for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "S", "MS", "M", "ML", "L"
+        """
+        return self._device.room_size
+
+    @property
+    def triggers(self) -> Dict[int, str]:
+        """
+        Return the triggers and their statuses for the device.
+
+        Only available if using Telnet.
+        """
+        return self._device.triggers
+
+    @property
+    def speaker_preset(self) -> Optional[int]:
+        """
+        Return the speaker preset for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "1", "2"
+        """
+        return self._device.speaker_preset
+
+    @property
+    def bt_transmitter(self) -> Optional[bool]:
+        """
+        Return the Bluetooth transmitter state for the device.
+
+        Only available if using Telnet.
+        """
+        return self._device.bt_transmitter
+
+    @property
+    def bt_output_mode(self) -> Optional[str]:
+        """
+        Return the Bluetooth output mode for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "Bluetooth + Speakers", "Bluetooth Only"
+        """
+        return self._device.bt_output_mode
+
+    @property
+    def delay_time(self) -> Optional[int]:
+        """
+        Return the delay time for the device in ms.
+
+        Only available if using Telnet.
+        """
+        return self._device.delay_time
+
+    @property
+    def audio_restorer(self) -> Optional[str]:
+        """
+        Return the audio restorer for the device.
+
+        Only available if using Telnet.
+
+        Possible values are: "Off", "Low", "Medium", "High"
+        """
+        return self._device.audio_restorer
+
+    @property
+    def graphic_eq(self) -> Optional[bool]:
+        """
+        Return the Graphic EQ status for the device.
+
+        Only available if using Telnet.
+        """
+        return self._device.graphic_eq
+
+    @property
+    def headphone_eq(self) -> Optional[bool]:
+        """
+        Return the Headphone EQ status for the device.
+
+        Only available if using Telnet.
+        """
+        return self._device.headphone_eq
+
+    ##########
+    # Getter #
+    ##########
+
+    def get_trigger(self, trigger: int) -> Optional[str]:
+        """
+        Return the status of a specific trigger.
+
+        Only available if using Telnet.
+
+        Valid trigger values are 1-3.
+        """
+        return self._device.get_trigger(trigger)
 
     ##########
     # Setter #
@@ -523,27 +766,31 @@ class DenonAVR(DenonAVRFoundation):
         """
         if not callable(async_client_getter):
             raise AvrCommandError("Provided object is not callable")
-        self._device.api.async_client_getter = async_client_getter
+        self._device.api.httpx_async_client.client_getter = async_client_getter
 
-    @run_async_synchronously(async_func=async_dynamic_eq_off)
-    def dynamic_eq_off(self) -> None:
+    async def async_dynamic_eq_off(self) -> None:
         """Turn DynamicEQ off."""
+        await self.audyssey.async_dynamiceq_off()
 
     async def async_dynamic_eq_on(self) -> None:
         """Turn DynamicEQ on."""
         await self.audyssey.async_dynamiceq_on()
 
-    @run_async_synchronously(async_func=async_dynamic_eq_on)
-    def dynamic_eq_on(self) -> None:
-        """Turn DynamicEQ on."""
-
     async def async_toggle_dynamic_eq(self) -> None:
         """Toggle DynamicEQ."""
         await self.audyssey.async_toggle_dynamic_eq()
 
-    @run_async_synchronously(async_func=async_toggle_dynamic_eq)
-    def toggle_dynamic_eq(self) -> None:
-        """Toggle DynamicEQ."""
+    async def async_set_multieq(self, value: str) -> None:
+        """Set MultiEQ mode."""
+        await self.audyssey.async_set_multieq(value)
+
+    async def async_set_reflevoffset(self, value: str) -> None:
+        """Set Reference Level Offset."""
+        await self.audyssey.async_set_reflevoffset(value)
+
+    async def async_set_dynamicvol(self, value: str) -> None:
+        """Set Dynamic Volume."""
+        await self.audyssey.async_set_dynamicvol(value)
 
     async def async_set_input_func(self, input_func: str) -> None:
         """
@@ -554,15 +801,6 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.input.async_set_input_func(input_func)
 
-    @run_async_synchronously(async_func=async_set_input_func)
-    def set_input_func(self, input_func: str) -> None:
-        """
-        Set input_func of device.
-
-        Valid values depend on the device and should be taken from
-        "input_func_list".
-        """
-
     async def async_set_sound_mode(self, sound_mode: str) -> None:
         """
         Set sound_mode of device.
@@ -572,87 +810,42 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.soundmode.async_set_sound_mode(sound_mode)
 
-    @run_async_synchronously(async_func=async_set_sound_mode)
-    def set_sound_mode(self, sound_mode: str) -> None:
-        """
-        Set sound_mode of device.
-
-        Valid values depend on the device and should be taken from
-        "sound_mode_list".
-        """
-
     async def async_toggle_play_pause(self) -> None:
         """Toggle play pause media player."""
         await self.input.async_toggle_play_pause()
 
-    @run_async_synchronously(async_func=async_toggle_play_pause)
-    def toggle_play_pause(self) -> None:
-        """Toggle play pause media player."""
-
     async def async_play(self) -> None:
         """Send play command to receiver command via HTTP post."""
         await self.input.async_play()
 
-    @run_async_synchronously(async_func=async_play)
-    def play(self) -> None:
-        """Send play command to receiver command via HTTP post."""
-
     async def async_pause(self) -> None:
         """Send pause command to receiver command via HTTP post."""
         await self.input.async_pause()
 
-    @run_async_synchronously(async_func=async_pause)
-    def pause(self) -> None:
-        """Send pause command to receiver command via HTTP post."""
-
     async def async_previous_track(self) -> None:
         """Send previous track command to receiver command via HTTP post."""
         await self.input.async_previous_track()
 
-    @run_async_synchronously(async_func=async_previous_track)
-    def previous_track(self) -> None:
-        """Send previous track command to receiver command via HTTP post."""
-
     async def async_next_track(self) -> None:
         """Send next track command to receiver command via HTTP post."""
         await self.input.async_next_track()
 
-    @run_async_synchronously(async_func=async_next_track)
-    def next_track(self) -> None:
-        """Send next track command to receiver command via HTTP post."""
-
     async def async_power_on(self) -> None:
         """Turn on receiver via HTTP get command."""
         await self._device.async_power_on()
 
-    @run_async_synchronously(async_func=async_power_on)
-    def power_on(self) -> None:
-        """Turn on receiver via HTTP get command."""
-
     async def async_power_off(self) -> None:
         """Turn off receiver via HTTP get command."""
         await self._device.async_power_off()
 
-    @run_async_synchronously(async_func=async_power_off)
-    def power_off(self) -> None:
-        """Turn off receiver via HTTP get command."""
-
     async def async_volume_up(self) -> None:
         """Volume up receiver via HTTP get command."""
         await self.vol.async_volume_up()
 
-    @run_async_synchronously(async_func=async_volume_up)
-    def volume_up(self) -> None:
-        """Volume up receiver via HTTP get command."""
-
     async def async_volume_down(self) -> None:
         """Volume down receiver via HTTP get command."""
         await self.vol.async_volume_down()
 
-    @run_async_synchronously(async_func=async_volume_down)
-    def volume_down(self) -> None:
-        """Volume down receiver via HTTP get command."""
-
     async def async_set_volume(self, volume: float) -> None:
         """
         Set receiver volume via HTTP get command.
@@ -662,39 +855,18 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.vol.async_set_volume(volume)
 
-    @run_async_synchronously(async_func=async_set_volume)
-    def set_volume(self, volume: float) -> None:
-        """
-        Set receiver volume via HTTP get command.
-
-        Volume is send in a format like -50.0.
-        Minimum is -80.0, maximum at 18.0
-        """
-
     async def async_mute(self, mute: bool) -> None:
         """Mute receiver via HTTP get command."""
         await self.vol.async_mute(mute)
 
-    @run_async_synchronously(async_func=async_mute)
-    def mute(self, mute: bool) -> None:
-        """Mute receiver via HTTP get command."""
-
     async def async_enable_tone_control(self) -> None:
         """Enable tone control to change settings like bass or treble."""
         await self.tonecontrol.async_enable_tone_control()
 
-    @run_async_synchronously(async_func=async_enable_tone_control)
-    def enable_tone_control(self) -> None:
-        """Enable tone control to change settings like bass or treble."""
-
     async def async_disable_tone_control(self) -> None:
         """Disable tone control to change settings like bass or treble."""
         await self.tonecontrol.async_disable_tone_control()
 
-    @run_async_synchronously(async_func=async_disable_tone_control)
-    def disable_tone_control(self) -> None:
-        """Disable tone control to change settings like bass or treble."""
-
     async def async_set_bass(self, value: int) -> None:
         """
         Set receiver bass.
@@ -706,17 +878,6 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.tonecontrol.async_set_bass(value)
 
-    @run_async_synchronously(async_func=async_set_bass)
-    def set_bass(self, value: int) -> None:
-        """
-        Set receiver bass.
-
-        Minimum is 0, maximum at 12
-
-        Note:
-        Doesn't work, if Dynamic Equalizer is active.
-        """
-
     async def async_bass_up(self) -> None:
         """
         Increase level of Bass.
@@ -726,15 +887,6 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.tonecontrol.async_bass_up()
 
-    @run_async_synchronously(async_func=async_bass_up)
-    def bass_up(self) -> None:
-        """
-        Increase level of Bass.
-
-        Note:
-        Doesn't work, if Dynamic Equalizer is active
-        """
-
     async def async_bass_down(self) -> None:
         """
         Decrease level of Bass.
@@ -744,15 +896,6 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.tonecontrol.async_bass_down()
 
-    @run_async_synchronously(async_func=async_bass_down)
-    def bass_down(self) -> None:
-        """
-        Decrease level of Bass.
-
-        Note:
-        Doesn't work, if Dynamic Equalizer is active
-        """
-
     async def async_set_treble(self, value: int) -> None:
         """
         Set receiver treble.
@@ -764,17 +907,6 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.tonecontrol.async_set_treble(value)
 
-    @run_async_synchronously(async_func=async_set_treble)
-    def set_treble(self, value: int) -> None:
-        """
-        Set receiver treble.
-
-        Minimum is 0, maximum at 12
-
-        Note:
-        Doesn't work, if Dynamic Equalizer is active.
-        """
-
     async def async_treble_up(self) -> None:
         """
         Increase level of Treble.
@@ -784,15 +916,6 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.tonecontrol.async_treble_up()
 
-    @run_async_synchronously(async_func=async_treble_up)
-    def treble_up(self) -> None:
-        """
-        Increase level of Treble.
-
-        Note:
-        Doesn't work, if Dynamic Equalizer is active
-        """
-
     async def async_treble_down(self) -> None:
         """
         Decrease level of Treble.
@@ -802,11 +925,288 @@ class DenonAVR(DenonAVRFoundation):
         """
         await self.tonecontrol.async_treble_down()
 
-    @run_async_synchronously(async_func=async_treble_down)
-    def treble_down(self) -> None:
-        """
-        Decrease level of Treble.
+    async def async_cursor_up(self) -> None:
+        """Send cursor up to receiver via HTTP get command."""
+        await self._device.async_cursor_up()
 
-        Note:
-        Doesn't work, if Dynamic Equalizer is active
+    async def async_cursor_down(self) -> None:
+        """Send cursor down to receiver via HTTP get command."""
+        await self._device.async_cursor_down()
+
+    async def async_cursor_left(self) -> None:
+        """Send cursor left to receiver via HTTP get command."""
+        await self._device.async_cursor_left()
+
+    async def async_cursor_right(self) -> None:
+        """Send cursor right to receiver via HTTP get command."""
+        await self._device.async_cursor_right()
+
+    async def async_cursor_enter(self) -> None:
+        """Send cursor enter to receiver via HTTP get command."""
+        await self._device.async_cursor_enter()
+
+    async def async_back(self) -> None:
+        """Send back to receiver via HTTP get command."""
+        await self._device.async_back()
+
+    async def async_info(self) -> None:
+        """Send info to receiver via HTTP get command."""
+        await self._device.async_info()
+
+    async def async_options(self) -> None:
+        """Raise options menu to receiver via HTTP get command."""
+        await self._device.async_options()
+
+    async def async_settings_menu(self) -> None:
+        """Raise settings menu to receiver via HTTP get command."""
+        await self._device.async_settings_menu()
+
+    async def async_channel_level_adjust(self) -> None:
+        """Toggle the channel level adjust menu on receiver via HTTP get command."""
+        await self._device.async_channel_level_adjust()
+
+    async def async_dimmer_toggle(self) -> None:
+        """Toggle dimmer on receiver via HTTP get command."""
+        await self._device.async_dimmer_toggle()
+
+    async def async_dimmer(self, mode: DimmerModes) -> None:
+        """Set dimmer mode on receiver via HTTP get command."""
+        await self._device.async_dimmer(mode)
+
+    async def async_auto_standby(self, auto_standby: AutoStandbys) -> None:
+        """Set auto standby on receiver via HTTP get command."""
+        await self._device.async_auto_standby(auto_standby)
+
+    async def async_sleep(self, sleep: Union[Literal["OFF"], int]) -> None:
         """
+        Set auto standby on receiver via HTTP get command.
+
+        Valid sleep values are "OFF" and 1-120 (in minutes)
+        """
+        await self._device.async_sleep(sleep)
+
+    async def async_delay_up(self) -> None:
+        """Increase delay of the audio."""
+        await self._device.async_delay_up()
+
+    async def async_delay_down(self) -> None:
+        """Decrease delay of the audio."""
+        await self._device.async_delay_down()
+
+    async def async_eco_mode(self, mode: EcoModes) -> None:
+        """Set Eco mode."""
+        await self._device.async_eco_mode(mode)
+
+    async def async_hdmi_output(self, output: HDMIOutputs) -> None:
+        """Set HDMI output."""
+        await self._device.async_hdmi_output(output)
+
+    async def async_hdmi_audio_decode(self, mode: HDMIAudioDecodes) -> None:
+        """Set HDMI Audio Decode mode on receiver via HTTP get command."""
+        await self._device.async_hdmi_audio_decode(mode)
+
+    async def async_video_processing_mode(self, mode: VideoProcessingModes) -> None:
+        """Set video processing mode on receiver via HTTP get command."""
+        await self._device.async_video_processing_mode(mode)
+
+    async def async_status(self) -> str:
+        """
+        Toggles the display of status on the device.
+
+        Only supported on Denon models.
+        """
+        return await self._device.async_status()
+
+    async def async_system_reset(self) -> None:
+        """DANGER! Reset the receiver via HTTP get command."""
+        await self._device.async_system_reset()
+
+    async def async_network_restart(self) -> None:
+        """Restart the network on the receiver via HTTP get command."""
+        await self._device.async_network_restart()
+
+    async def async_speaker_preset(self, preset: int) -> None:
+        """
+        Set speaker preset on receiver via HTTP get command.
+
+        Valid preset values are 1-2.
+        """
+        await self._device.async_speaker_preset(preset)
+
+    async def async_speaker_preset_toggle(self) -> None:
+        """
+        Toggle speaker preset on receiver via HTTP get command.
+
+        Only available if using Telnet.
+        """
+        await self._device.async_speaker_preset_toggle()
+
+    async def async_bt_transmitter_on(self) -> None:
+        """Turn on Bluetooth transmitter on receiver via HTTP get command."""
+        await self._device.async_bt_transmitter_on()
+
+    async def async_bt_transmitter_off(self) -> None:
+        """Turn off Bluetooth transmitter on receiver via HTTP get command."""
+        await self._device.async_bt_transmitter_off()
+
+    async def async_bt_transmitter_toggle(self) -> None:
+        """
+        Toggle Bluetooth transmitter mode on receiver via HTTP get command.
+
+        Only available if using Telnet.
+        """
+        await self._device.async_bt_transmitter_toggle()
+
+    async def async_bt_output_mode(self, mode: BluetoothOutputModes) -> None:
+        """Set Bluetooth transmitter mode on receiver via HTTP get command."""
+        await self._device.async_bt_output_mode(mode)
+
+    async def async_bt_output_mode_toggle(self) -> None:
+        """
+        Toggle Bluetooth output mode on receiver via HTTP get command.
+
+        Only available if using Telnet.
+        """
+        await self._device.async_bt_output_mode_toggle()
+
+    async def async_delay_time_up(self) -> None:
+        """Delay time up on receiver via HTTP get command."""
+        await self._device.async_delay_time_up()
+
+    async def async_delay_time_down(self) -> None:
+        """Delay time up on receiver via HTTP get command."""
+        await self._device.async_delay_time_down()
+
+    async def async_delay_time(self, delay_time: int) -> None:
+        """
+        Set delay time on receiver via HTTP get command.
+
+        :param delay_time: Delay time in ms. Valid values are 0-999.
+        """
+        await self._device.async_delay_time(delay_time)
+
+    async def async_audio_restorer(self, mode: AudioRestorers):
+        """Set audio restorer on receiver via HTTP get command."""
+        await self._device.async_audio_restorer(mode)
+
+    async def async_remote_control_lock(self):
+        """Set remote control lock on receiver via HTTP get command."""
+        await self._device.async_remote_control_lock()
+
+    async def async_remote_control_unlock(self):
+        """Set remote control unlock on receiver via HTTP get command."""
+        await self._device.async_remote_control_unlock()
+
+    async def async_panel_lock(self, panel_lock_mode: PanelLocks):
+        """Set panel lock on receiver via HTTP get command."""
+        await self._device.async_panel_lock(panel_lock_mode)
+
+    async def async_panel_unlock(self):
+        """Set panel unlock on receiver via HTTP get command."""
+        await self._device.async_panel_unlock()
+
+    async def async_graphic_eq_on(self) -> None:
+        """Turn on Graphic EQ on receiver via HTTP get command."""
+        await self._device.async_graphic_eq_on()
+
+    async def async_graphic_eq_off(self) -> None:
+        """Turn off Graphic EQ on receiver via HTTP get command."""
+        await self._device.async_graphic_eq_off()
+
+    async def async_graphic_eq_toggle(self) -> None:
+        """
+        Toggle Graphic EQ on receiver via HTTP get command.
+
+        Only available if using Telnet.
+        """
+        await self._device.async_graphic_eq_toggle()
+
+    async def async_headphone_eq_on(self) -> None:
+        """Turn on Headphone EQ on receiver via HTTP get command."""
+        await self._device.async_headphone_eq_on()
+
+    async def async_headphone_eq_off(self) -> None:
+        """Turn off Headphone EQ on receiver via HTTP get command."""
+        await self._device.async_headphone_eq_off()
+
+    async def async_headphone_eq_toggle(self) -> None:
+        """
+        Toggle Headphone EQ on receiver via HTTP get command.
+
+        Only available if using Telnet.
+        """
+        await self._device.async_headphone_eq_toggle()
+
+    async def async_tactile_transducer_on(self) -> None:
+        """Turn on tactile transducer on receiver via HTTP get command."""
+        await self._device.async_tactile_transducer_on()
+
+    async def async_tactile_transducer_off(self) -> None:
+        """Turn on tactile transducer on receiver via HTTP get command."""
+        await self._device.async_tactile_transducer_off()
+
+    async def async_tactile_transducer_toggle(self) -> None:
+        """
+        Turn on tactile transducer on receiver via HTTP get command.
+
+        Only available if using Telnet.
+        """
+        await self._device.async_tactile_transducer_toggle()
+
+    async def async_tactile_transducer_level_up(self) -> None:
+        """Increase the transducer level on receiver via HTTP get command."""
+        await self._device.async_tactile_transducer_level_up()
+
+    async def async_tactile_transducer_level_down(self) -> None:
+        """Decrease the transducer on receiver via HTTP get command."""
+        await self._device.async_tactile_transducer_level_down()
+
+    async def async_transducer_lpf(self, lpf: TransducerLPFs) -> None:
+        """Set transducer low pass filter on receiver via HTTP get command."""
+        await self._device.async_transducer_lpf(lpf)
+
+    async def async_room_size(self, room_size: RoomSizes) -> None:
+        """Set room size on receiver via HTTP get command."""
+        await self._device.async_room_size(room_size)
+
+    async def async_trigger_on(self, trigger: int) -> None:
+        """
+        Set trigger to ON on receiver via HTTP get command.
+
+        :param trigger: Trigger number to set to ON. Valid values are 1-3.
+        """
+        await self._device.async_trigger_on(trigger)
+
+    async def async_trigger_off(self, trigger: int) -> None:
+        """
+        Set trigger to OFF on receiver via HTTP get command.
+
+        :param trigger: Trigger number to set to OFF. Valid values are 1-3.
+        """
+        await self._device.async_trigger_off(trigger)
+
+    async def async_trigger_toggle(self, trigger: int) -> None:
+        """
+        Toggle trigger on receiver via HTTP get command.
+
+        Only available if using Telnet.
+
+        :param trigger: Trigger number to toggle. Valid values are 1-3.
+        """
+        await self._device.async_trigger_toggle(trigger)
+
+    async def async_quick_select_mode(self, quick_select_number: int) -> None:
+        """
+        Set quick select mode on receiver via HTTP get command.
+
+        :param quick_select_number: Quick select number to set. Valid values are 1-5.
+        """
+        await self._device.async_quick_select_mode(quick_select_number)
+
+    async def async_quick_select_memory(self, quick_select_number: int) -> None:
+        """
+        Set quick select memory on receiver via HTTP get command.
+
+        :param quick_select_number: Quick select number to set. Valid values are 1-5.
+        """
+        await self._device.async_quick_select_memory(quick_select_number)
