@@ -39,6 +39,7 @@ from .const import (
     DIMMER_MODE_MAP_LABELS,
     ECO_MODE_MAP,
     ECO_MODE_MAP_LABELS,
+    HDMI_OUTPUT_MAP,
     HDMI_OUTPUT_MAP_LABELS,
     MAIN_ZONE,
     POWER_STATES,
@@ -153,6 +154,7 @@ class DenonAVRDeviceInfo:
     _dimmer: Optional[str] = attr.ib(
         converter=attr.converters.optional(str), default=None
     )
+    _dimmer_modes = get_args(DimmerModes)
     _auto_standby: Optional[str] = attr.ib(
         converter=attr.converters.optional(str), default=None
     )
@@ -166,9 +168,11 @@ class DenonAVRDeviceInfo:
     _eco_mode: Optional[str] = attr.ib(
         converter=attr.converters.optional(str), default=None
     )
+    _eco_modes = get_args(EcoModes)
     _hdmi_output: Optional[str] = attr.ib(
         converter=attr.converters.optional(str), default=None
     )
+    _hdmi_outputs = get_args(HDMIOutputs)
     _hdmi_audio_decode: Optional[str] = attr.ib(
         converter=attr.converters.optional(str), default=None
     )
@@ -308,8 +312,8 @@ class DenonAVRDeviceInfo:
         self, zone: str, event: str, parameter: str
     ) -> None:
         """Handle a delay change event."""
-        if event == "PS" and parameter[0:3] == "DEL":
-            self._delay = int(parameter[4:])
+        if event == "PS" and parameter[0:5] == "DELAY":
+            self._delay = int(parameter[6:])
 
     async def _async_eco_mode_callback(
         self, zone: str, event: str, parameter: str
@@ -385,7 +389,8 @@ class DenonAVRDeviceInfo:
         self, zone: str, event: str, parameter: str
     ) -> None:
         """Handle a delay time change event."""
-        if event != "PS" or parameter[0:3] != "DEL":
+        # do not match "DELAY" as it's another event
+        if event != "PS" or parameter[0:3] != "DEL" or parameter[0:5] == "DELAY":
             return
 
         self._delay_time = int(parameter[4:])
@@ -1199,7 +1204,7 @@ class DenonAVRDeviceInfo:
 
     async def async_dimmer(self, mode: DimmerModes) -> None:
         """Set dimmer mode on receiver via HTTP get command."""
-        if mode not in DimmerModes:
+        if mode not in self._dimmer_modes:
             raise AvrCommandError("Invalid dimmer mode")
 
         mapped_mode = DIMMER_MODE_MAP[mode]
@@ -1397,15 +1402,19 @@ class DenonAVRDeviceInfo:
             raise AvrCommandError("Quick select number must be between 1 and 5")
 
         if self.telnet_available:
+            if "denon" in self.manufacturer.lower():
+                command = self.telnet_commands.command_quick_select_memory
+            else:
+                command = self.telnet_commands.command_smart_select_memory
             await self.telnet_api.async_send_commands(
-                self.telnet_commands.command_quick_select_mode.format(
-                    number=quick_select_number
-                )
+                command.format(number=quick_select_number)
             )
         else:
-            await self.api.async_get_command(
-                self.urls.command_quick_select_mode.format(number=quick_select_number)
-            )
+            if "denon" in self.manufacturer.lower():
+                command = self.urls.command_quick_select_mode
+            else:
+                command = self.urls.command_smart_select_mode
+            await self.api.async_get_command(command.format(number=quick_select_number))
 
     async def async_quick_select_memory(self, quick_select_number: int) -> None:
         """
@@ -1417,15 +1426,19 @@ class DenonAVRDeviceInfo:
             raise AvrCommandError("Quick select number must be between 1 and 5")
 
         if self.telnet_available:
+            if "denon" in self.manufacturer.lower():
+                command = self.telnet_commands.command_quick_select_mode
+            else:
+                command = self.telnet_commands.command_smart_select_mode
             await self.telnet_api.async_send_commands(
-                self.telnet_commands.command_quick_select_memory.format(
-                    number=quick_select_number
-                )
+                command.format(number=quick_select_number)
             )
         else:
-            await self.api.async_get_command(
-                self.urls.command_quick_select_memory.format(number=quick_select_number)
-            )
+            if "denon" in self.manufacturer.lower():
+                command = self.urls.command_quick_select_memory
+            else:
+                command = self.urls.command_smart_select_memory
+            await self.api.async_get_command(command.format(number=quick_select_number))
 
     async def async_delay_up(self) -> None:
         """Delay up on receiver via HTTP get command."""
@@ -1447,7 +1460,7 @@ class DenonAVRDeviceInfo:
 
     async def async_eco_mode(self, mode: EcoModes) -> None:
         """Set Eco mode."""
-        if mode not in EcoModes:
+        if mode not in self._eco_modes:
             raise AvrCommandError("Invalid Eco mode")
 
         mapped_mode = ECO_MODE_MAP[mode]
@@ -1462,10 +1475,10 @@ class DenonAVRDeviceInfo:
 
     async def async_hdmi_output(self, output: HDMIOutputs) -> None:
         """Set HDMI output."""
-        if output not in HDMIOutputs:
+        if output not in self._hdmi_outputs:
             raise AvrCommandError("Invalid HDMI output mode")
 
-        mapped_output = HDMIOutputs[output]
+        mapped_output = HDMI_OUTPUT_MAP[output]
         if self.telnet_available:
             await self.telnet_api.async_send_commands(
                 self.telnet_commands.command_hdmi_output.format(output=mapped_output)
