@@ -8,13 +8,14 @@ This module implements the handler for input functions of Denon AVR receivers.
 """
 
 import asyncio
-import html
 import logging
+from collections.abc import Hashable
 from copy import deepcopy
-from typing import Dict, Hashable, List, Optional, Tuple
+from typing import Dict, List, Optional, Set, Tuple
 
 import attr
 import httpx
+from ftfy import fix_text
 
 from .appcommand import AppCommands
 from .const import (
@@ -56,15 +57,15 @@ def lower_string(value: Optional[str]) -> Optional[str]:
     return str(value).lower()
 
 
-def unescape_string(value: Optional[str]) -> Optional[str]:
-    """Perform HTML unescape on value."""
+def fix_string(value: Optional[str]) -> Optional[str]:
+    """Fix errors in string like unescaped HTML and wrong utf-8 encoding."""
     if value is None:
         return value
-    return html.unescape(str(value))
+    return fix_text(str(value)).strip()
 
 
 def set_input_func(
-    instance: DenonAVRFoundation, attribute: attr.Attribute, value: str
+    instance: "DenonAVRInput", attribute: attr.Attribute, value: str
 ) -> str:
     """Set input_func after mapping from input_func_map."""
     # pylint: disable=protected-access
@@ -75,8 +76,11 @@ def set_input_func(
     # AirPlay and Internet Radio are not always listed in available sources
     if value in ["AirPlay", "Internet Radio"]:
         if value not in instance._input_func_map:
+            instance._additional_input_funcs.add(value)
             instance._input_func_map[value] = value
             instance._input_func_map_rev[value] = value
+            instance._netaudio_func_list.append(value)
+            instance._playing_func_list.append(value)
     try:
         input_func = instance._input_func_map_rev[value]
     except KeyError:
@@ -121,6 +125,12 @@ class DenonAVRInput(DenonAVRFoundation):
         ),
         default=attr.Factory(list),
     )
+    _additional_input_funcs: Set[str] = attr.ib(
+        validator=attr.validators.deep_iterable(
+            attr.validators.instance_of(str), attr.validators.instance_of(set)
+        ),
+        default=attr.Factory(set),
+    )
     _media_update_handle: asyncio.TimerHandle = attr.ib(default=None)
     _input_func_update_lock: asyncio.Lock = attr.ib(default=attr.Factory(asyncio.Lock))
 
@@ -134,22 +144,22 @@ class DenonAVRInput(DenonAVRFoundation):
     )
 
     _artist: Optional[str] = attr.ib(
-        converter=attr.converters.optional(unescape_string), default=None
+        converter=attr.converters.optional(fix_string), default=None
     )
     _album: Optional[str] = attr.ib(
-        converter=attr.converters.optional(unescape_string), default=None
+        converter=attr.converters.optional(fix_string), default=None
     )
     _band: Optional[str] = attr.ib(
-        converter=attr.converters.optional(unescape_string), default=None
+        converter=attr.converters.optional(fix_string), default=None
     )
     _title: Optional[str] = attr.ib(
-        converter=attr.converters.optional(unescape_string), default=None
+        converter=attr.converters.optional(fix_string), default=None
     )
     _frequency: Optional[str] = attr.ib(
-        converter=attr.converters.optional(unescape_string), default=None
+        converter=attr.converters.optional(fix_string), default=None
     )
     _station: Optional[str] = attr.ib(
-        converter=attr.converters.optional(unescape_string), default=None
+        converter=attr.converters.optional(fix_string), default=None
     )
     _image_url: Optional[str] = attr.ib(
         converter=attr.converters.optional(str), default=None
@@ -158,6 +168,13 @@ class DenonAVRInput(DenonAVRFoundation):
         converter=attr.converters.optional(str), default=None
     )
 
+    _renamed_sources_warnings: Set[Tuple[str, str]] = attr.ib(
+        validator=attr.validators.deep_iterable(
+            attr.validators.instance_of(tuple), attr.validators.instance_of(set)
+        ),
+        default=attr.Factory(set),
+    )
+
     # Update tags for attributes
     # AppCommand.xml interface
     appcommand_attrs = {AppCommands.GetAllZoneSource: None}
@@ -175,9 +192,16 @@ class DenonAVRInput(DenonAVRFoundation):
         for tag in self.appcommand_attrs:
             self._device.api.add_appcommand_update_tag(tag)
 
+        power_event = "ZM"
+        if self._device.zone == ZONE2:
+            power_event = "Z2"
+        elif self._device.zone == ZONE3:
+            power_event = "Z3"
+        self._device.telnet_api.register_callback(
+            power_event, self._async_power_callback
+        )
         self._device.telnet_api.register_callback("SI", self._async_input_callback)
-        self._device.telnet_api.register_callback("PW", self._async_power_callback)
-        self._device.telnet_api.register_callback("NS", self._async_netaudio_callback)
+        self._device.telnet_api.register_callback("NSE", self._async_netaudio_callback)
         self._device.telnet_api.register_callback("TF", self._async_tuner_callback)
         self._device.telnet_api.register_callback("HD", self._async_hdtuner_callback)
         self._device.telnet_api.register_callback(
@@ -198,7 +222,7 @@ class DenonAVRInput(DenonAVRFoundation):
         if self._device.power != POWER_ON:
             return
 
-        if self._schedule_media_updates() is True:
+        if self._schedule_media_updates():
             self._state = STATE_PLAYING
         else:
             self._unset_media_state()
@@ -215,7 +239,7 @@ class DenonAVRInput(DenonAVRFoundation):
             self._stop_media_update()
             self._unset_media_state()
             self._state = STATE_OFF
-        elif self._schedule_media_updates() is True:
+        elif self._schedule_media_updates():
             self._state = STATE_PLAYING
         else:
             self._unset_media_state()
@@ -255,8 +279,11 @@ class DenonAVRInput(DenonAVRFoundation):
 
     def _update_netaudio(self) -> None:
         """Update netaudio information."""
-        self._device.telnet_api.send_commands("NSE")
-        self._schedule_netaudio_update()
+        if self._device.telnet_available:
+            self._device.telnet_api.send_commands("NSE", skip_confirmation=True)
+            self._schedule_netaudio_update()
+        else:
+            self._stop_media_update()
 
     async def _async_netaudio_callback(
         self, zone: str, event: str, parameter: str
@@ -267,12 +294,12 @@ class DenonAVRInput(DenonAVRFoundation):
         if self._input_func not in self._netaudio_func_list:
             return
 
-        if parameter.startswith("E1"):
-            self._title = parameter[2:].strip()
-        elif parameter.startswith("E2"):
-            self._artist = parameter[2:].strip()
-        elif parameter.startswith("E4"):
-            self._album = parameter[2:].strip()
+        if parameter.startswith("1"):
+            self._title = parameter[1:]
+        elif parameter.startswith("2"):
+            self._artist = parameter[1:]
+        elif parameter.startswith("4"):
+            self._album = parameter[1:]
         self._band = None
         self._frequency = None
         self._station = None
@@ -283,7 +310,7 @@ class DenonAVRInput(DenonAVRFoundation):
             port=self._device.api.port,
             hash=hash((self._title, self._artist, self._album)),
         )
-        await self._async_test_image_accessable()
+        await self._async_test_image_accessible()
 
     def _schedule_tuner_update(self) -> None:
         """Schedule a tuner update task."""
@@ -295,8 +322,13 @@ class DenonAVRInput(DenonAVRFoundation):
 
     def _update_tuner(self) -> None:
         """Update tuner information."""
-        self._device.telnet_api.send_commands("TFAN?", "TFANNAME?")
-        self._schedule_tuner_update()
+        if self._device.telnet_available:
+            self._device.telnet_api.send_commands(
+                "TFAN?", "TFANNAME?", skip_confirmation=True
+            )
+            self._schedule_tuner_update()
+        else:
+            self._stop_media_update()
 
     async def _async_tuner_callback(
         self, zone: str, event: str, parameter: str
@@ -308,9 +340,9 @@ class DenonAVRInput(DenonAVRFoundation):
             return
 
         if parameter.startswith("ANNAME"):
-            self._station = parameter[6:].strip()
+            self._station = parameter[6:]
         elif len(parameter) == 8:
-            self._frequency = "{}.{}".format(parameter[2:6], parameter[6:]).strip("0")
+            self._frequency = f"{parameter[2:6]}.{parameter[6:]}".strip("0")
             if parameter[2:] > "050000":
                 self._band = "AM"
             else:
@@ -324,7 +356,7 @@ class DenonAVRInput(DenonAVRFoundation):
         self._image_url = STATIC_ALBUM_URL.format(
             host=self._device.api.host, port=self._device.api.port
         )
-        await self._async_test_image_accessable()
+        await self._async_test_image_accessible()
 
     def _schedule_hdtuner_update(self) -> None:
         """Schedule a HD tuner update task."""
@@ -336,8 +368,11 @@ class DenonAVRInput(DenonAVRFoundation):
 
     def _update_hdtuner(self) -> None:
         """Update HD tuner information."""
-        self._device.telnet_api.send_commands("HD?")
-        self._schedule_hdtuner_update()
+        if self._device.telnet_available:
+            self._device.telnet_api.send_commands("HD?", skip_confirmation=True)
+            self._schedule_hdtuner_update()
+        else:
+            self._stop_media_update()
 
     async def _async_hdtuner_callback(
         self, zone: str, event: str, parameter: str
@@ -349,13 +384,13 @@ class DenonAVRInput(DenonAVRFoundation):
             return
 
         if parameter.startswith("ARTIST"):
-            self._artist = parameter[6:].strip()
+            self._artist = parameter[6:]
         elif parameter.startswith("TITLE"):
-            self._title = parameter[5:].strip()
+            self._title = parameter[5:]
         elif parameter.startswith("ALBUM"):
-            self._album = parameter[5:].strip()
+            self._album = parameter[5:]
         elif parameter.startswith("ST NAME"):
-            self._station = parameter[7:].strip()
+            self._station = parameter[7:]
 
         self._band = None
         self._frequency = None
@@ -364,13 +399,13 @@ class DenonAVRInput(DenonAVRFoundation):
         self._image_url = STATIC_ALBUM_URL.format(
             host=self._device.api.host, port=self._device.api.port
         )
-        await self._async_test_image_accessable()
+        await self._async_test_image_accessible()
 
     async def _async_input_func_update_callback(
         self, zone: str, event: str, parameter: str
     ) -> None:
         """Handle input func update events."""
-        if self._input_func_update_lock.locked() is True:
+        if self._input_func_update_lock.locked():
             return
         async with self._input_func_update_lock:
             await self.async_update_inputfuncs()
@@ -379,18 +414,23 @@ class DenonAVRInput(DenonAVRFoundation):
         self, global_update: bool = False, cache_id: Optional[Hashable] = None
     ) -> None:
         """Update input functions asynchronously."""
+        _LOGGER.debug("Starting input update")
         # Ensure instance is setup before updating
-        if self._is_setup is False:
+        if not self._is_setup:
             self.setup()
 
         # Update input functions
+        _LOGGER.debug("Updating input functions")
         await self.async_update_inputfuncs(
             global_update=global_update, cache_id=cache_id
         )
         # Update state
+        _LOGGER.debug("Updating input state")
         await self.async_update_state(global_update=global_update, cache_id=cache_id)
         # Update media state
+        _LOGGER.debug("Updating media state")
         await self.async_update_media_state(cache_id=cache_id)
+        _LOGGER.debug("Finished input update")
 
     async def async_get_sources_deviceinfo(self) -> Dict[str, str]:
         """Get sources from Deviceinfo.xml."""
@@ -400,7 +440,7 @@ class DenonAVRInput(DenonAVRFoundation):
                 self._device.urls.deviceinfo, cache_id=id(self._device)
             )
         except AvrRequestError as err:
-            _LOGGER.debug("Error when getting sources", exc_info=err)
+            _LOGGER.debug("Error when getting sources: %s", err)
             raise
 
         receiver_sources = {}
@@ -425,9 +465,9 @@ class DenonAVRInput(DenonAVRFoundation):
                 # Get list of all input sources of receiver
                 xml_list = xml_zonecapa.find("./InputSource/List")
                 for xml_source in xml_list.findall("Source"):
-                    receiver_sources[
-                        xml_source.find("FuncName").text
-                    ] = xml_source.find("DefaultName").text
+                    receiver_sources[xml_source.find("FuncName").text] = (
+                        xml_source.find("DefaultName").text
+                    )
 
         return receiver_sources
 
@@ -458,13 +498,12 @@ class DenonAVRInput(DenonAVRFoundation):
                     self._device.urls.appcommand, tags, cache_id=cache_id
                 )
         except AvrRequestError as err:
-            _LOGGER.debug("Error when getting changed sources", exc_info=err)
+            _LOGGER.debug("Error when getting changed sources: %s", err)
             raise
 
         for child in xml.findall(
-            "./cmd[@{attribute}='{cmd}']/functionrename/list".format(
-                attribute=APPCOMMAND_CMD_TEXT, cmd=AppCommands.GetRenameSource.cmd_text
-            )
+            f"./cmd[@{APPCOMMAND_CMD_TEXT}='{AppCommands.GetRenameSource.cmd_text}']"
+            "/functionrename/list"
         ):
             try:
                 renamed_sources[child.find("name").text.strip()] = child.find(
@@ -474,9 +513,8 @@ class DenonAVRInput(DenonAVRFoundation):
                 continue
 
         for child in xml.findall(
-            "./cmd[@{attribute}='{cmd}']/functiondelete/list".format(
-                attribute=APPCOMMAND_CMD_TEXT, cmd=AppCommands.GetDeletedSource.cmd_text
-            )
+            f"./cmd[@{APPCOMMAND_CMD_TEXT}='{AppCommands.GetDeletedSource.cmd_text}']"
+            "/functiondelete/list"
         ):
             try:
                 deleted_sources[child.find("FuncName").text.strip()] = (
@@ -485,6 +523,8 @@ class DenonAVRInput(DenonAVRFoundation):
             except AttributeError:
                 continue
 
+        self._replace_duplicate_sources(renamed_sources)
+
         return (renamed_sources, deleted_sources)
 
     async def async_get_changed_sources_status_xml(
@@ -519,12 +559,11 @@ class DenonAVRInput(DenonAVRFoundation):
                 )
             else:
                 raise AvrProcessingError(
-                    "Method does not work for receiver type {}".format(
-                        self._device.receiver.type
-                    )
+                    "Method does not work for receiver type"
+                    f" {self._device.receiver.type}"
                 )
         except AvrRequestError as err:
-            _LOGGER.debug("Error when getting changed sources", exc_info=err)
+            _LOGGER.debug("Error when getting changed sources: %s", err)
             raise
 
         # Get the relevant tags from XML structure
@@ -574,6 +613,8 @@ class DenonAVRInput(DenonAVRFoundation):
             except IndexError:
                 _LOGGER.error("List of deleted sources incomplete, continuing anyway")
 
+        self._replace_duplicate_sources(renamed_sources)
+
         return (renamed_sources, deleted_sources)
 
     async def async_update_inputfuncs(
@@ -608,9 +649,13 @@ class DenonAVRInput(DenonAVRFoundation):
                 "Receiver sources list empty. Please check if device is powered on."
             )
 
+        # Add additional input functions discovered at run-time
+        for function in self._additional_input_funcs:
+            receiver_sources[function] = function
+
         # Get renamed and deleted sources
         # From Appcommand.xml if supported
-        if self._device.use_avr_2016_update is True:
+        if self._device.use_avr_2016_update:
             (
                 renamed_sources,
                 deleted_sources,
@@ -630,7 +675,7 @@ class DenonAVRInput(DenonAVRFoundation):
                 receiver_sources[key] = value
 
         # Remove all deleted sources
-        if self._show_all_inputs is False:
+        if not self._show_all_inputs:
             for deleted_source in deleted_sources.items():
                 if deleted_source[1] == "DEL":
                     receiver_sources.pop(deleted_source[0], None)
@@ -642,7 +687,7 @@ class DenonAVRInput(DenonAVRFoundation):
         playing_func_list = []
 
         for item in receiver_sources.items():
-            # Mapping of item[0] because some func names are inconsistant
+            # Mapping of item[0] because some func names are inconsistent
             # at AVR-X receivers
 
             m_item_0 = SOURCE_MAPPING.get(item[0], item[0])
@@ -689,8 +734,12 @@ class DenonAVRInput(DenonAVRFoundation):
             deleted_sources,
         ) = await self.async_get_changed_sources_status_xml(cache_id=cache_id)
 
+        # Add additional input functions discovered at run-time
+        for function in self._additional_input_funcs:
+            receiver_sources[function] = function
+
         # Remove all deleted sources
-        if self._show_all_inputs is False:
+        if not self._show_all_inputs:
             for deleted_source in deleted_sources.items():
                 if deleted_source[1] == "DEL":
                     receiver_sources.pop(deleted_source[0], None)
@@ -719,21 +768,22 @@ class DenonAVRInput(DenonAVRFoundation):
         self, global_update: bool = False, cache_id: Optional[Hashable] = None
     ):
         """Update state of device."""
-        if self._device.use_avr_2016_update is True:
+        if self._device.use_avr_2016_update is None:
+            raise AvrProcessingError(
+                "Device is not setup correctly, update method not set"
+            )
+
+        if self._device.use_avr_2016_update:
             await self.async_update_attrs_appcommand(
                 self.appcommand_attrs, global_update=global_update, cache_id=cache_id
             )
-        elif self._device.use_avr_2016_update is False:
+        else:
             urls = [self._device.urls.status]
             if self._device.zone == MAIN_ZONE:
                 urls.append(self._device.urls.mainzone)
             await self.async_update_attrs_status_xml(
                 self.status_xml_attrs, urls, cache_id=cache_id
             )
-        else:
-            raise AvrProcessingError(
-                "Device is not setup correctly, update method not set"
-            )
 
     async def async_update_media_state(self, cache_id: Optional[Hashable] = None):
         """Update media state of device."""
@@ -815,15 +865,18 @@ class DenonAVRInput(DenonAVRFoundation):
             # On track change assume device is PLAYING
             self._state = STATE_PLAYING
 
-        await self._async_test_image_accessable()
+        await self._async_test_image_accessible()
 
-    async def _async_test_image_accessable(self) -> None:
-        """Test if image URL is accessable."""
+    async def _async_test_image_accessible(self) -> None:
+        """Test if image URL is accessible."""
         if self._image_available is None and self._image_url is not None:
-            client = self._device.api.async_client_getter()
+            client = self._device.api.httpx_async_client.client_getter()
             try:
                 res = await client.get(
-                    self._image_url, timeout=self._device.api.timeout
+                    self._image_url,
+                    timeout=httpx.Timeout(
+                        self._device.api.timeout, read=self._device.api.read_timeout
+                    ),
                 )
                 res.raise_for_status()
             except httpx.TimeoutException:
@@ -839,10 +892,10 @@ class DenonAVRInput(DenonAVRFoundation):
                 self._image_available = True
             finally:
                 # Close the default AsyncClient but keep custom clients open
-                if self._device.api.is_default_async_client():
+                if self._device.api.httpx_async_client.is_default_async_client():
                     await client.aclose()
         # Already tested that image URL is not accessible
-        elif self._image_available is False:
+        elif not self._image_available:
             self._image_url = None
 
     def _unset_media_state(self) -> None:
@@ -855,6 +908,33 @@ class DenonAVRInput(DenonAVRFoundation):
         self._station = None
         self._image_url = None
 
+    def _replace_duplicate_sources(self, sources: Dict[str, str]) -> None:
+        """Replace duplicate renamed sources (values) with their original names."""
+        seen_values = set()
+        duplicate_values = set()
+
+        for value in sources.values():
+            if value in seen_values:
+                duplicate_values.add(value)
+            seen_values.add(value)
+
+        for duplicate in duplicate_values:
+            for key, value in sources.items():
+                if value == duplicate:
+                    sources[key] = key
+
+                    if (key, value) not in self._renamed_sources_warnings:
+                        _LOGGER.warning(
+                            (
+                                "Input source '%s' is renamed to non-unique name '%s'. "
+                                "Using original name. Please choose unique names in "
+                                "your receiver's web-interface"
+                            ),
+                            key,
+                            value,
+                        )
+                        self._renamed_sources_warnings.add((key, value))
+
     ##############
     # Properties #
     ##############
@@ -959,20 +1039,24 @@ class DenonAVRInput(DenonAVRFoundation):
                 direct_mapping = False
         # AVR-nonX receiver and if no mapping was found get parameter for
         # setting input_func directly
-        if direct_mapping is True:
+        if direct_mapping:
             try:
                 linp = self._input_func_map[input_func]
             except KeyError as err:
                 raise AvrCommandError(
-                    "No mapping for input source {}".format(input_func)
+                    f"No mapping for input source {input_func}"
                 ) from err
         # Create command URL and send command via HTTP GET
         if linp in self._favorite_func_list:
             command_url = self._device.urls.command_fav_src + linp
+            telnet_command = self._device.telnet_commands.command_fav_src + linp
         else:
             command_url = self._device.urls.command_sel_src + linp
-
-        await self._device.api.async_get_command(command_url)
+            telnet_command = self._device.telnet_commands.command_sel_src + linp
+        if self._device.telnet_available:
+            await self._device.telnet_api.async_send_commands(telnet_command)
+        else:
+            await self._device.api.async_get_command(command_url)
 
     async def async_toggle_play_pause(self) -> None:
         """Toggle play pause media player."""
@@ -993,15 +1077,26 @@ class DenonAVRInput(DenonAVRFoundation):
             if self._state == STATE_PLAYING:
                 _LOGGER.info("Already playing, play command not sent")
                 return
-
-            await self._device.api.async_get_command(self._device.urls.command_play)
+            if self._device.telnet_available:
+                await self._device.telnet_api.async_send_commands(
+                    self._device.telnet_commands.command_play
+                )
+            else:
+                await self._device.api.async_get_command(self._device.urls.command_play)
             self._state = STATE_PLAYING
 
     async def async_pause(self) -> None:
         """Send pause command to receiver command via HTTP post."""
         # Use pause command only for sources which support NETAUDIO
         if self._input_func in self._netaudio_func_list:
-            await self._device.api.async_get_command(self._device.urls.command_pause)
+            if self._device.telnet_available:
+                await self._device.telnet_api.async_send_commands(
+                    self._device.telnet_commands.command_play
+                )
+            else:
+                await self._device.api.async_get_command(
+                    self._device.urls.command_pause
+                )
             self._state = STATE_PAUSED
 
     async def async_previous_track(self) -> None:
