@@ -42,6 +42,7 @@ TESTING_RECEIVERS = {
     "AVR-3312": (NO_ZONES, denonavr.const.AVR),
     "NR1609": (ZONE2, denonavr.const.AVR_X_2016),
     "AVC-8500H": (ZONE2_ZONE3, denonavr.const.AVR_X_2016),
+    "AVC-A10H": (ZONE2_ZONE3, denonavr.const.AVR_X_2016),
     "AVR-X4300H": (ZONE2_ZONE3, denonavr.const.AVR_X_2016),
     "AVR-X1100W": (ZONE2, denonavr.const.AVR_X),
     "SR6012": (ZONE2, denonavr.const.AVR_X_2016),
@@ -70,9 +71,7 @@ DESCRIPTION_URL2 = "/upnp/desc/aios_device/aios_device.xml"
 
 def get_sample_content(filename):
     """Return sample content form file."""
-    with open(
-        "tests/xml/{filename}".format(filename=filename), encoding="utf-8"
-    ) as file:
+    with open(f"tests/xml/{filename}", encoding="utf-8") as file:
         return file.read()
 
 
@@ -93,61 +92,49 @@ class TestMainFunctions:
         try:
             if request.url.path == STATUS_URL:
                 content = get_sample_content(
-                    "{receiver}-formMainZone_MainZoneXmlStatus{port}"
-                    ".xml".format(receiver=self.testing_receiver, port=port_suffix)
+                    f"{self.testing_receiver}-formMainZone_MainZoneXmlStatus"
+                    f"{port_suffix}.xml"
                 )
             elif request.url.path == STATUS_Z2_URL:
                 content = get_sample_content(
-                    "{receiver}-formZone2_Zone2XmlStatus{port}.xml".format(
-                        receiver=self.testing_receiver, port=port_suffix
-                    )
+                    f"{self.testing_receiver}-formZone2_Zone2XmlStatus{port_suffix}.xml"
                 )
             elif request.url.path == STATUS_Z3_URL:
                 content = get_sample_content(
-                    "{receiver}-formZone3_Zone3XmlStatus{port}.xml".format(
-                        receiver=self.testing_receiver, port=port_suffix
-                    )
+                    f"{self.testing_receiver}-formZone3_Zone3XmlStatus{port_suffix}.xml"
                 )
             elif request.url.path == MAINZONE_URL:
                 content = get_sample_content(
-                    "{receiver}-formMainZone_MainZoneXml{port}.xml".format(
-                        receiver=self.testing_receiver, port=port_suffix
-                    )
+                    f"{self.testing_receiver}-formMainZone_MainZoneXml{port_suffix}.xml"
                 )
             elif request.url.path == DEVICEINFO_URL:
                 content = get_sample_content(
-                    "{receiver}-Deviceinfo{port}.xml".format(
-                        receiver=self.testing_receiver, port=port_suffix
-                    )
+                    f"{self.testing_receiver}-Deviceinfo{port_suffix}.xml"
                 )
             elif request.url.path == NETAUDIOSTATUS_URL:
                 content = get_sample_content(
-                    "{receiver}-formNetAudio_StatusXml{port}.xml".format(
-                        receiver=self.testing_receiver, port=port_suffix
-                    )
+                    f"{self.testing_receiver}-formNetAudio_StatusXml{port_suffix}.xml"
                 )
             elif request.url.path == TUNERSTATUS_URL:
                 content = get_sample_content(
-                    "{receiver}-formTuner_TunerXml{port}.xml".format(
-                        receiver=self.testing_receiver, port=port_suffix
-                    )
+                    f"{self.testing_receiver}-formTuner_TunerXml{port_suffix}.xml"
                 )
             elif request.url.path == HDTUNERSTATUS_URL:
                 content = get_sample_content(
-                    "{receiver}-formTuner_HdXml{port}.xml".format(
-                        receiver=self.testing_receiver, port=port_suffix
-                    )
+                    f"{self.testing_receiver}-formTuner_HdXml{port_suffix}.xml"
                 )
             elif request.url.path == APPCOMMAND_URL:
                 content_str = request.read().decode("utf-8")
                 if "GetFriendlyName" in content_str:
                     ep_suffix = "-setup"
-                else:
+                elif "GetAllZoneSource" in content_str:
                     ep_suffix = "-update"
+                elif "GetSurroundModeStatus" in content_str:
+                    ep_suffix = "-update-soundmode"
+                else:
+                    ep_suffix = "-update-tonecontrol"
                 content = get_sample_content(
-                    "{receiver}-AppCommand{ep}{port}.xml".format(
-                        receiver=self.testing_receiver, port=port_suffix, ep=ep_suffix
-                    )
+                    f"{self.testing_receiver}-AppCommand{ep_suffix}{port_suffix}.xml"
                 )
             elif request.url.path in [DESCRIPTION_URL1, DESCRIPTION_URL2]:
                 content = get_sample_content("AVR-X1600H_upnp.xml")
@@ -167,27 +154,27 @@ class TestMainFunctions:
         self.future.set_result(True)
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_receiver_type(self, httpx_mock: HTTPXMock):
         """Check that receiver type is determined correctly."""
         httpx_mock.add_callback(self.custom_matcher)
         for receiver, spec in TESTING_RECEIVERS.items():
-            print("Receiver: {}".format(receiver))
+            print(f"Receiver: {receiver}")
             # Switch receiver and update to load new sample files
             self.testing_receiver = receiver
             self.denon = denonavr.DenonAVR(FAKE_IP, add_zones=spec[0])
             await self.denon.async_setup()
-            assert (
-                self.denon.receiver_type == spec[1].type
-            ), "Receiver type is {} not {} for receiver {}".format(
-                self.denon.receiver_type, spec[1].type, receiver
+            assert self.denon.receiver_type == spec[1].type, (
+                f"Receiver type is {self.denon.receiver_type} not {spec[1].type} for"
+                f" receiver {receiver}"
             )
-            assert (
-                self.denon.receiver_port == spec[1].port
-            ), "Receiver port is {} not {} for receiver {}".format(
-                self.denon.receiver_port, spec[1].port, receiver
+            assert self.denon.receiver_port == spec[1].port, (
+                f"Receiver port is {self.denon.receiver_port} not {spec[1].port} for"
+                f" receiver {receiver}"
             )
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_input_func_switch(self, httpx_mock: HTTPXMock):
         """Switch through all input functions of all tested receivers."""
         httpx_mock.add_callback(self.custom_matcher)
@@ -197,37 +184,37 @@ class TestMainFunctions:
             self.denon = denonavr.DenonAVR(FAKE_IP, add_zones=spec[0])
             # Switch through all functions and check if successful
             for name, zone in self.denon.zones.items():
-                print("Receiver: {}, Zone: {}".format(receiver, name))
+                print(f"Receiver: {receiver}, Zone: {name}")
                 await self.denon.zones[name].async_update()
                 assert len(zone.input_func_list) > 0
                 for input_func in zone.input_func_list:
                     await self.denon.zones[name].async_set_input_func(input_func)
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_attributes_not_none(self, httpx_mock: HTTPXMock):
         """Check that certain attributes are not None."""
         httpx_mock.add_callback(self.custom_matcher)
         for receiver, spec in TESTING_RECEIVERS.items():
-            print("Receiver: {}".format(receiver))
+            print(f"Receiver: {receiver}")
             # Switch receiver and update to load new sample files
             self.testing_receiver = receiver
             self.denon = denonavr.DenonAVR(FAKE_IP, add_zones=spec[0])
             await self.denon.async_setup()
-            assert self.denon.name is not None, "Name is None for receiver {}".format(
-                receiver
-            )
+            assert self.denon.name is not None, f"Name is None for receiver {receiver}"
             assert (
                 self.denon.support_sound_mode is not None
-            ), "support_sound_mode is None for receiver {}".format(receiver)
+            ), f"support_sound_mode is None for receiver {receiver}"
             await self.denon.async_update()
             assert (
                 self.denon.power is not None
-            ), "Power status is None for receiver {}".format(receiver)
-            assert self.denon.state is not None, "State is None for receiver {}".format(
-                receiver
-            )
+            ), f"Power status is None for receiver {receiver}"
+            assert (
+                self.denon.state is not None
+            ), f"State is None for receiver {receiver}"
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_sound_mode(self, httpx_mock: HTTPXMock):
         """Check if a valid sound mode is returned."""
         httpx_mock.add_callback(self.custom_matcher)
@@ -237,7 +224,7 @@ class TestMainFunctions:
             self.denon = denonavr.DenonAVR(FAKE_IP, add_zones=spec[0])
             # Switch through all functions and check if successful
             for name in self.denon.zones:
-                print("Receiver: {}, Zone: {}".format(receiver, name))
+                print(f"Receiver: {receiver}, Zone: {name}")
                 await self.denon.zones[name].async_update()
                 support_sound_mode = self.denon.zones[name].support_sound_mode
                 sound_mode = self.denon.zones[name].sound_mode
@@ -250,23 +237,32 @@ class TestMainFunctions:
     async def test_protocol_connected(self):
         """Connected after connection made."""
         proto = DenonAVRTelnetProtocol(None, None)
-        transport = AsyncMock()
+        transport = mock.Mock(is_closing=lambda: False)
         proto.connection_made(transport)
 
-        assert proto.connected is True
+        assert proto.connected
 
     @pytest.mark.asyncio
     async def test_protocol_not_connected(self):
         """Not connected when connection not made."""
         proto = DenonAVRTelnetProtocol(None, None)
 
-        assert proto.connected is False
+        assert not proto.connected
+
+    @pytest.mark.asyncio
+    async def test_protocol_closing(self):
+        """Not connected when connection is closing."""
+        proto = DenonAVRTelnetProtocol(None, None)
+        transport = mock.Mock(is_closing=lambda: True)
+        proto.connection_made(transport)
+
+        assert not proto.connected
 
     @pytest.mark.asyncio
     async def test_protocol_writes_to_transport(self):
         """Write writes to transport."""
         proto = DenonAVRTelnetProtocol(None, None)
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         proto.connection_made(transport)
 
         proto.write("abc")
@@ -289,7 +285,7 @@ class TestMainFunctions:
         """Read callback triggered for CR terminated message."""
         data_received_mock = mock.Mock()
         proto = DenonAVRTelnetProtocol(data_received_mock, None)
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         proto.connection_made(transport)
         proto.data_received(b"abc\r")
 
@@ -300,7 +296,7 @@ class TestMainFunctions:
         """Read callback not triggered for non CR terminated message."""
         data_received_mock = mock.Mock()
         proto = DenonAVRTelnetProtocol(data_received_mock, None)
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         proto.connection_made(transport)
         proto.data_received(b"abc")
 
@@ -311,7 +307,7 @@ class TestMainFunctions:
         """Connection lost triggers callback."""
         conn_lost_mock = mock.Mock()
         proto = DenonAVRTelnetProtocol(None, conn_lost_mock)
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         proto.connection_made(transport)
         proto.connection_lost(Exception())
 
@@ -326,7 +322,7 @@ class TestMainFunctions:
         api._protocol = protocol
         protocol.connected = False
 
-        assert api.healthy is False
+        assert not api.healthy
 
     @pytest.mark.asyncio
     async def test_telnetapi_healthy_if_protocol_connected(self):
@@ -337,7 +333,7 @@ class TestMainFunctions:
         api._protocol = protocol
         protocol.connected = True
 
-        assert api.healthy is True
+        assert api.healthy
 
     @pytest.mark.asyncio
     async def test_register_callback_invalid_event_raises_valueerror(self):
@@ -366,7 +362,7 @@ class TestMainFunctions:
         """Connect raises NetworkError when ConnectionRefused."""
         api = DenonAVRTelnetApi()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=ConnectionRefusedError()
             )
             with pytest.raises(AvrNetworkError):
@@ -377,7 +373,9 @@ class TestMainFunctions:
         """Connect raises NetworkError when OSError."""
         api = DenonAVRTelnetApi()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(side_effect=OSError())
+            debug_mock.return_value.create_connection = mock.AsyncMock(
+                side_effect=OSError()
+            )
             with pytest.raises(AvrNetworkError):
                 await api.async_connect()
 
@@ -386,7 +384,9 @@ class TestMainFunctions:
         """Connect raises NetworkError when IOError."""
         api = DenonAVRTelnetApi()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(side_effect=IOError())
+            debug_mock.return_value.create_connection = mock.AsyncMock(
+                side_effect=IOError()
+            )
             with pytest.raises(AvrNetworkError):
                 await api.async_connect()
 
@@ -395,16 +395,17 @@ class TestMainFunctions:
         """Connect raises AvrTimeoutError when TimeoutError."""
         api = DenonAVRTelnetApi()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=asyncio.TimeoutError()
             )
             with pytest.raises(AvrTimoutError):
                 await api.async_connect()
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_receive_callback_called(self, httpx_mock: HTTPXMock):
         """Check that the callback is triggered whena message is received."""
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         protocol = DenonAVRTelnetProtocol(None, None)
 
         def create_conn(proto_lambda, host, port):
@@ -413,14 +414,18 @@ class TestMainFunctions:
             protocol._on_connection_lost = proto._on_connection_lost
             # pylint: disable=protected-access
             protocol._on_message = proto._on_message
+            proto.connection_made(transport)
+            protocol.connection_made(transport)
             return [transport, proto]
 
         httpx_mock.add_callback(self.custom_matcher)
 
         self.denon = denonavr.DenonAVR(FAKE_IP)
+        # pylint: disable=protected-access
+        self.denon._device.telnet_api._send_confirmation_timeout = 0.1
         await self.denon.async_setup()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=create_conn
             )
             await self.denon.async_telnet_connect()
@@ -433,26 +438,30 @@ class TestMainFunctions:
             mock_obj.method.assert_called_once()
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_mute_on(self, httpx_mock: HTTPXMock):
         """Check that mute on is processed."""
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         protocol = DenonAVRTelnetProtocol(None, None)
 
         def create_conn(proto_lambda, host, port):
             proto = proto_lambda()
             # pylint: disable=protected-access
             protocol._on_message = proto._on_message
+            proto.connection_made(transport)
+            protocol.connection_made(transport)
             return [transport, proto]
 
         httpx_mock.add_callback(self.custom_matcher)
         self.denon = denonavr.DenonAVR(FAKE_IP)
+        # pylint: disable=protected-access
+        self.denon._device.telnet_api._send_confirmation_timeout = 0.1
         await self.denon.async_setup()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=create_conn
             )
             await self.denon.async_telnet_connect()
-            protocol.connection_made(transport)
             self.future = asyncio.Future()
             self.denon.register_callback("ALL", self._callback)
             protocol.data_received(b"MUON\r")
@@ -460,26 +469,30 @@ class TestMainFunctions:
             assert self.denon.muted
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_mute_off(self, httpx_mock: HTTPXMock):
         """Check that mute off is processed."""
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         protocol = DenonAVRTelnetProtocol(None, None)
 
         def create_conn(proto_lambda, host, port):
             proto = proto_lambda()
             # pylint: disable=protected-access
             protocol._on_message = proto._on_message
+            proto.connection_made(transport)
+            protocol.connection_made(transport)
             return [transport, proto]
 
         httpx_mock.add_callback(self.custom_matcher)
         self.denon = denonavr.DenonAVR(FAKE_IP)
+        # pylint: disable=protected-access
+        self.denon._device.telnet_api._send_confirmation_timeout = 0.1
         await self.denon.async_setup()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=create_conn
             )
             await self.denon.async_telnet_connect()
-            protocol.connection_made(transport)
             self.future = asyncio.Future()
             self.denon.register_callback("ALL", self._callback)
             protocol.data_received(b"MUOFF\r")
@@ -487,80 +500,92 @@ class TestMainFunctions:
             assert not self.denon.muted
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_power_on(self, httpx_mock: HTTPXMock):
         """Check that power on is processed."""
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         protocol = DenonAVRTelnetProtocol(None, None)
 
         def create_conn(proto_lambda, host, port):
             proto = proto_lambda()
             # pylint: disable=protected-access
             protocol._on_message = proto._on_message
+            proto.connection_made(transport)
+            protocol.connection_made(transport)
             return [transport, proto]
 
         httpx_mock.add_callback(self.custom_matcher)
         self.denon = denonavr.DenonAVR(FAKE_IP)
+        # pylint: disable=protected-access
+        self.denon._device.telnet_api._send_confirmation_timeout = 0.1
         await self.denon.async_setup()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=create_conn
             )
             await self.denon.async_telnet_connect()
-            protocol.connection_made(transport)
             self.future = asyncio.Future()
             self.denon.register_callback("ALL", self._callback)
-            protocol.data_received(b"PWON\r")
+            protocol.data_received(b"ZMON\r")
             await self.future
             assert self.denon.power == "ON"
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_power_off(self, httpx_mock: HTTPXMock):
         """Check that power off is processed."""
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         protocol = DenonAVRTelnetProtocol(None, None)
 
         def create_conn(proto_lambda, host, port):
             proto = proto_lambda()
             # pylint: disable=protected-access
             protocol._on_message = proto._on_message
+            proto.connection_made(transport)
+            protocol.connection_made(transport)
             return [transport, proto]
 
         httpx_mock.add_callback(self.custom_matcher)
         self.denon = denonavr.DenonAVR(FAKE_IP)
+        # pylint: disable=protected-access
+        self.denon._device.telnet_api._send_confirmation_timeout = 0.1
         await self.denon.async_setup()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=create_conn
             )
             await self.denon.async_telnet_connect()
-            protocol.connection_made(transport)
             self.future = asyncio.Future()
             self.denon.register_callback("ALL", self._callback)
-            protocol.data_received(b"PWSTANDBY\r")
+            protocol.data_received(b"ZMOFF\r")
             await self.future
-            assert self.denon.power == "STANDBY"
+            assert self.denon.power == "OFF"
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_volume_min(self, httpx_mock: HTTPXMock):
         """Check that minimum volume is processed."""
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         protocol = DenonAVRTelnetProtocol(None, None)
 
         def create_conn(proto_lambda, host, port):
             proto = proto_lambda()
             # pylint: disable=protected-access
             protocol._on_message = proto._on_message
+            proto.connection_made(transport)
+            protocol.connection_made(transport)
             return [transport, proto]
 
         httpx_mock.add_callback(self.custom_matcher)
         self.denon = denonavr.DenonAVR(FAKE_IP)
+        # pylint: disable=protected-access
+        self.denon._device.telnet_api._send_confirmation_timeout = 0.1
         await self.denon.async_setup()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=create_conn
             )
             await self.denon.async_telnet_connect()
-            protocol.connection_made(transport)
             self.future = asyncio.Future()
             self.denon.register_callback("ALL", self._callback)
             protocol.data_received(b"MV00\r")
@@ -568,26 +593,30 @@ class TestMainFunctions:
             assert self.denon.volume == -80.0
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_volume_wholenumber(self, httpx_mock: HTTPXMock):
         """Check that whole number volume is processed."""
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         protocol = DenonAVRTelnetProtocol(None, None)
 
         def create_conn(proto_lambda, host, port):
             proto = proto_lambda()
             # pylint: disable=protected-access
             protocol._on_message = proto._on_message
+            proto.connection_made(transport)
+            protocol.connection_made(transport)
             return [transport, proto]
 
         httpx_mock.add_callback(self.custom_matcher)
         self.denon = denonavr.DenonAVR(FAKE_IP)
+        # pylint: disable=protected-access
+        self.denon._device.telnet_api._send_confirmation_timeout = 0.1
         await self.denon.async_setup()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=create_conn
             )
             await self.denon.async_telnet_connect()
-            protocol.connection_made(transport)
             self.future = asyncio.Future()
             self.denon.register_callback("ALL", self._callback)
             protocol.data_received(b"MV56\r")
@@ -595,37 +624,32 @@ class TestMainFunctions:
             assert self.denon.volume == -24.0
 
     @pytest.mark.asyncio
+    @pytest.mark.httpx_mock(can_send_already_matched_responses=True)
     async def test_volume_fraction(self, httpx_mock: HTTPXMock):
         """Check that fractional volume is processed."""
-        transport = mock.Mock()
+        transport = mock.Mock(is_closing=lambda: False)
         protocol = DenonAVRTelnetProtocol(None, None)
 
         def create_conn(proto_lambda, host, port):
             proto = proto_lambda()
             # pylint: disable=protected-access
             protocol._on_message = proto._on_message
+            proto.connection_made(transport)
+            protocol.connection_made(transport)
             return [transport, proto]
 
         httpx_mock.add_callback(self.custom_matcher)
         self.denon = denonavr.DenonAVR(FAKE_IP)
+        # pylint: disable=protected-access
+        self.denon._device.telnet_api._send_confirmation_timeout = 0.1
         await self.denon.async_setup()
         with mock.patch("asyncio.get_event_loop", new_callable=mock.Mock) as debug_mock:
-            debug_mock.return_value.create_connection = AsyncMock(
+            debug_mock.return_value.create_connection = mock.AsyncMock(
                 side_effect=create_conn
             )
             await self.denon.async_telnet_connect()
-            protocol.connection_made(transport)
             self.future = asyncio.Future()
             self.denon.register_callback("ALL", self._callback)
             protocol.data_received(b"MV565\r")
             await self.future
             assert self.denon.volume == -23.5
-
-
-class AsyncMock(mock.MagicMock):
-    """Mocking async methods compatible to python 3.7."""
-
-    # pylint: disable=invalid-overridden-method,useless-super-delegation
-    async def __call__(self, *args, **kwargs):
-        """Call."""
-        return super().__call__(*args, **kwargs)
